/************************************************************************************************
Sernder Standheizung mit Handy Steuern
                                                                                                *
Bentigte Hardware:
    AT Mega 8L
	Siemens M35I                  ( RXD / TXD)
    Sender Standheizung VW T91    (  )
                                                                                                *
Der uC wird mit dem Akku vom Handy versorgt, daher kein Pegelwandler notwendig.
                                                                                                *
Nach dem Einstecken werden Alle SMS Gelscht !
Es Darf kein PIN im Handy vergeben sein.
                                                                                                *
Es werden alle X Minuten die SMS ausgewertet.
                                                                                                *
Nach erhalt einer SMS mit dem Inhalt "1" wird ein Impuls an Heizung Ein gegeben.
Nach erhalt einer SMS mit dem Inhalt "0" wird ein Impuls an Heizung Aus gegeben.
                                                                                                *
Die Status LED Wird ausgewertet, wenn Heizung OK: Kurzer Rckruf,
sonst Status SMS.
                                                                                                *
Letzte nderung: 11 / 2011
************************************************************************************************/

#ifndef F_CPU
#define F_CPU 7378200
#endif
#define UART_BAUD 19200
#define UART_BAUD_CALC(UART_BAUD,F_OSC) ((F_CPU)/((UART_BAUD)*16L)-1)

#include <avr/io.h>        // Definition der Ein und Ausgnge
#include <avr/interrupt.h> // interrupt ermglichen

// #include <util/setbaud.h>  // Baudrate Berechnen lassen
#include <util/delay.h>    // einfache kurze verzgerungsschleifen
#include <stdint.h>        // erweiterte Variablen  Definirern
// #include <inttypes.h>
// #include <avr/sleep.h>		   // Schlafmodi Ansprechen




#define UART_MAXSTRLEN 200
volatile uint8_t uart_str_complete = 0;     // 1 .. String komplett empfangen
volatile uint8_t uart_str_count = 0;
volatile char uart_string[UART_MAXSTRLEN + 1] = "";






void ports_init()
{
  DDRD  = (1<<6) | (1<<7);					 // Ausgnge Festlegen
  DDRB  = (1<<0);
  ACSR = 0x80;								// Analogcomparator ausschalten
}


void uart_init()
{
  UBRRH=(unsigned char)(UART_BAUD_CALC(UART_BAUD,F_CPU)>>8);
  UBRRL=(unsigned char)UART_BAUD_CALC(UART_BAUD,F_CPU);


  UCSRB |= (1<<RXEN) | (1<<TXEN) | (1<<RXCIE);  // UART RX, TX und RX Interrupt einschalten
  UCSRC = (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0); // Asynchron 8N1

  // Flush Receive-Buffer (entfernen evtl. vorhandener ungltiger Werte)
  do
  {
    uint8_t dummy;
    (void) (dummy = UDR);
  }
  while (UCSRA & (1 << RXC));
}


void long_delay(uint16_t ms)
{
  for (; ms>0; ms--) _delay_ms(1);
}

void e_long_delay(uint16_t ms)
{
  for(; ms>0; ms--) _delay_ms(10);
}

static inline int
uart_putc (const uint8_t c)
{
  while (!(UCSRA & (1 << UDRE)));  // Warten, bis UDR bereit ist fr einen neuen Wert
  UDR = c;  // UDR schreiben startet die bertragung
  return 1;
}


void uart_puts (const char *s)
{
  while (*s)
  {
    // so lange *s != '\0' also ungleich dem "String-Endezeichen(Terminator)"
    uart_putc(*s);
    s++;
  }
  uart_putc(0x0D);
}

void fone_call ()
{
  uart_puts("atd015152000000;");
  e_long_delay(1000);       // Warten in 10 ms
  uart_puts("AT+CHUP");	  // Auflegen
}



ISR(USART_RXC_vect)
{
  unsigned char nextChar;

  nextChar = UDR;   // Daten aus dem Puffer lesen
  if( uart_str_complete == 0 )  	// wenn uart_string gerade in Verwendung, neues Zeichen verwerfen
  {
    // Daten werden erst in uart_string geschrieben,
    // wenn nicht String-Ende/max Zeichenlnge erreicht ist/string gerade verarbeitet wird
    if( nextChar != '\n' &&
        nextChar != '\r' &&
        uart_str_count < UART_MAXSTRLEN - 1 )
    {
      uart_string[uart_str_count] = nextChar;
      uart_str_count++;
    }
    else
    {
      uart_string[uart_str_count] = '\0';
      uart_str_count = 0;
      uart_str_complete = 1;
    }
  }
}

//****************************************************************************************************************

void delete_all_sms()
{

#define DELETE_WAIT_TIME 50 // Zeit pro 10ms
#define SMS_MEMORY 20

  uint16_t timeabboard = 0;
  for( int sms_count	= 1; sms_count<SMS_MEMORY+1 ; sms_count++)
  {
    uart_str_complete = 0;
    char s[3];

    uart_puts("AT+CMGD=" + itoa( sms_count, s, 10 )); // Variableninhalt als Ascii senden !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    for( timeabboard = 0; timeabboard != DELETE_WAIT_TIME; timeabboard++) // Warten Bis Handy OK / ERROR Gesendet hat
    {
      long_delay(1);
      if(uart_str_complete ==1)
      {
        timeabboard = DELETE_WAIT_TIME-1;
      }
    }
  }
}

//***************************************************************************************************************



int main(void)
{
  ports_init();
  uart_init();
  sei();

  while(1)
  {
    PORTD ^= ( 1 << PB6 );  // Toggle PB0 z.&nbsp;B. angeschlossene LED
    e_long_delay(80);       // Warten in 10 ms
    //fone_call();
    //e_long_delay(100);       // Warten in 10 ms
    delete_all_sms();

    if (uart_str_complete == 1)
    {
      uart_puts(uart_string);
      uart_str_complete = 0;
    }
  }
  return 0;
}


