/************************************************************************************************
Sernder Standheizung mit Handy Steuern
                                                                                                *
Bentigte Hardware:
    AT Mega 8L
	Siemens M35I                  ( RXD PD0 PIN 3, TXD PD1 PIN 4 )
    Sender Standheizung VW T91    ( OFF PD6 PIN 12, ON PD7 PIN 13 )                             *
	Status LED                    ( PB0 PIN 14 )
    Quarz 7,3782 MHZ              ( PB6 PB7 PIN 9 PIN 10 )                                                                                                
																								*
Der uC wird mit dem Akku vom Handy versorgt, daher kein Pegelwandler notwendig.
                                                                                                *
Nach dem Einstecken werden alle SMS gelscht !
Es Darf kein PIN im Handy vergeben sein.
                                                                                                *
Es werden alle X Minuten die SMS ausgewertet.
                                                                                                *
Nach erhalt einer SMS mit dem Inhalt "1" wird ein Impuls an Heizung Ein gegeben.
Nach erhalt einer SMS mit dem Inhalt "0" wird ein Impuls an Heizung Aus gegeben.
                                                                                                *
TODO: Die Status LED des Senders wird ausgewertet, wenn OK: 
Kurzer Rckruf
                                                                                                *
Letzte nderung: 11 / 2011
************************************************************************************************/

#ifndef F_CPU
#define F_CPU 7378200
#endif
#define UART_BAUD 19200
#define UART_BAUD_CALC(UART_BAUD,F_OSC) ((F_CPU)/((UART_BAUD)*16L)-1)

#include <avr/io.h>        // Definition der Ein und Ausgnge
#include <avr/interrupt.h> // interrupt ermglichen

// #include <util/setbaud.h>  // Baudrate Berechnen lassen
#include <util/delay.h>    // einfache kurze verzgerungsschleifen
#include <stdint.h>        // erweiterte Variablen  Definirern
// #include <inttypes.h>
 #include <avr/sleep.h>		   // Schlafmodi Ansprechen

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CALL_WAIT_TIME   1200 // Zeit von anrufen bis auflegen in 10ms
#define DELETE_WAIT_TIME    8 // SMS Lschen Wartezeit in 100ms
#define SMS_MEMORY         20 // Anzahl SMS Speicherpltze

#define UART_MAXSTRLEN_0 20  // Standard Uart
#define UART_MAXSTRLEN_1 200 // Fs SMS Bentigt, Wenn Uart 0 buffer voll ist
volatile uint8_t uart_str_complete_0 = 0;     // 1 .. String komplett empfangen
volatile uint8_t uart_str_count_0 = 0;
volatile char uart_string_0[UART_MAXSTRLEN_0 + 1];
volatile uint8_t uart_str_complete_1 = 0;     // 2 .. String komplett empfangen
volatile uint8_t uart_str_count_1 = 0;
volatile char uart_string_1[UART_MAXSTRLEN_1 + 1];

volatile unsigned char handynr[30];


void ports_init()
{
  DDRD  = (1<<6) | (1<<7);	 // Ausgnge Festlegen
  DDRB  = (1<<0);
  ACSR  = (1<<ACD);			 // Analogcomparator ausschalten
}


void uart_init()
{
  UBRRH=(unsigned char)(UART_BAUD_CALC(UART_BAUD,F_CPU)>>8);
  UBRRL=(unsigned char)UART_BAUD_CALC(UART_BAUD,F_CPU);

  UCSRB |= (1<<RXEN) | (1<<TXEN) | (1<<RXCIE);  // UART RX, TX und RX Interrupt einschalten
  UCSRC = (1<<URSEL) | (1<<UCSZ1) | (1<<UCSZ0); // Asynchron 8N1

  // Flush Receive-Buffer (entfernen evtl. vorhandener ungltiger Werte)
  do
  {
    uint8_t dummy;
    (void) (dummy = UDR);
  }
  while (UCSRA & (1 << RXC));
}


int uart_putc (const uint8_t c)
{
  while (!(UCSRA & (1 << UDRE)));  // Warten, bis UDR bereit ist fr einen neuen Wert
  UDR = c;  // UDR schreiben startet die bertragung
  return 1;
}


void uart_puts (const char *s)
{
  while (*s)
  {
    // so lange *s != '\0' also ungleich dem "String-Endezeichen(Terminator)"
    uart_putc(*s);
    s++;
  }
}


ISR(USART_RXC_vect)
{
  unsigned char nextChar;

  nextChar = UDR;                   // Daten aus dem Puffer lesen
  if( uart_str_complete_0 == 0 )  	// wenn uart_string_0 gerade in Verwendung, neues Zeichen in uart_string_1 schreiben
  {
    // Daten werden erst in uart_string_0 geschrieben,
    // wenn nicht String-Ende/max Zeichenlnge erreicht ist/string gerade verarbeitet wird
    if( nextChar != '\n' &&
        nextChar != 0x0A &&
        nextChar != 0x0D &&
        uart_str_count_0 < UART_MAXSTRLEN_0 - 1 )
    {
      uart_string_0[uart_str_count_0] = nextChar;
      uart_str_count_0++;
    }
    else
    {
      if (uart_str_count_0 != 0)        // Wenn kein Inhalt sondern nur Enter gesendet wird nichts tun.
      {
        uart_string_0[uart_str_count_0] = '\0';
        uart_str_count_0 = 0;
        uart_str_complete_0 = 1;
      }
    }
  }
  else if( uart_str_complete_1 == 0 )  	// wenn uart_string_1 gerade in Verwendung, neues Zeichen verwerfen
  {
    // Daten werden erst in uart_string_0 geschrieben,
    // wenn nicht String-Ende/max Zeichenlnge erreicht ist/string gerade verarbeitet wird
    if( nextChar != '\n' &&
        nextChar != 0x0A &&
        nextChar != 0x0D &&
        uart_str_count_1 < UART_MAXSTRLEN_1 - 1 )
    {
      uart_string_1[uart_str_count_1] = nextChar;
      uart_str_count_1++;
    }
    else
    {
      if (uart_str_count_1 != 0)        // Wenn kein Inhalt sondern nur Enter gesendet wird nichts tun.
      {
        uart_string_1[uart_str_count_1] = '\0';
        uart_str_count_1 = 0;
        uart_str_complete_1 = 1;
      }
    }
  }

}


void long_delay(uint16_t ms)
{
  for (; ms>0; ms--) _delay_ms(1);
}


void e_long_delay(uint16_t ms)
{
  for(; ms>0; ms--) _delay_ms(10);
}


int intelli_wait(uint16_t maxwaittime, uint8_t Uartselect, char exitword[20] )// Max Wartezeit in ms, 0 = Uart0 1 = Uart1, Erwartetes Wort
{
  int for_exit = 0;
  if (Uartselect == 0)
  {
    for (uint16_t i = 0; (i < maxwaittime) && (for_exit !=1); i++)   //
    {
      long_delay(1);
      if ( uart_str_complete_0 == 1 &&
           strcmp (uart_string_0 , exitword) == 0 )
      {
        for_exit = 1;
      }
    }
  }
  else
  {
    for (uint16_t i = 0; (i < maxwaittime) && (for_exit !=1); i++)   //
    {
      long_delay(1);
      if ( uart_str_complete_1 == 1 &&
           strcmp (uart_string_1 , exitword) == 0 )
      {
        for_exit = 1;
      }
    }
  }
  return for_exit;
}


int htoi( const char *s )
{
  return (int)strtol( s, (char **)NULL, 16 );
}


void sms_nummerauslesen()
{
  unsigned char i;
  uint8_t for_exit = 0;
  uint64_t temp_1= 0;
  //char Buffer[20];

  strncpy(handynr, uart_string_1 , 2);  // Wie Lang ist SMC Nummer ?
  temp_1 = htoi(handynr);
  temp_1 = (temp_1 * 2) +4; // zzgl. MSG Flags etc.

  //uart_puts("Laenge SMSC\r");
  //uart_puts(itoa(temp_1,Buffer,10));
  //uart_putc(0x0D);

  strncpy(handynr, uart_string_1 + temp_1, 2); // SMS Nummer-Lnge auslesen

  //uart_puts("Laenge SMS Nr.\r");
  //uart_puts(handynr);

  strncpy(handynr, uart_string_1 + temp_1+4, htoi(handynr)+1); // SMS Nummer auslesen

  //uart_puts("Handy Nr.unbearbeitet\r");
  //uart_puts(handynr);

  for (i = 0; (i < 20) && (for_exit !=1); i++)   // Zahlen Verkehrtherum ausgeben  9451--> 4915
  {
    i++;
    if(handynr[i] != 'F')
    {
      uart_putc(handynr[i]);
    }
    else
    {
      for_exit = 1;
    }
    i--;
    if(handynr[i] != 'F')
    {
      uart_putc(handynr[i]);
    }
    else
    {
      for_exit = 1;
    }
    i++;
  }
}


void fone_call ()
{
  uart_puts("atd+");
  sms_nummerauslesen();
  uart_puts(";\r");
  e_long_delay(CALL_WAIT_TIME);
  uart_puts("AT+CHUP\r");	  // Auflegen
}


void delete_all_sms()
{


  for( int sms_count	= 1; sms_count<SMS_MEMORY+1 ; sms_count++)
  {
    uart_str_complete_0 = 0;
    char s[3];

    uart_puts("AT+CMGD=");
    uart_puts(itoa( sms_count, s, 10 )); // Variableninhalt als Ascii senden
    uart_putc(0x0D);
    intelli_wait(2000,0,"OK");
  }
}


int check_sms()
{
  uint8_t strleng;
  uart_str_complete_0 = 0;
  uart_str_complete_1 = 0;
  uart_puts("\rAT+CMGR=1\r");
  if (  intelli_wait(5000,1,"OK") != 1) //Wenn SMS Vorhanden
  {
    uart_puts("AT+CMGD= 1\r"); // SMS lschen

    strleng =strlen(uart_string_1) - 4; // SMS Textinhalt =0 ?
    if (strcmp (uart_string_1 + strleng,"0130") == 0)
    {
      PORTD |= ( 1 << PB6 );
      e_long_delay(300);
      PORTD &= ~( 1 << PB6 );
      fone_call();
      return 1;
    }
    strleng =strlen(uart_string_1) - 4; // SMS Textinhalt =1 ?
    if (strcmp (uart_string_1 + strleng,"0131") == 0)
    {
      PORTD |= ( 1 << PB7 );
      e_long_delay(300);
      PORTD &= ~( 1 << PB7 );
      fone_call();
      return 1;
    }
  }
  else
  {
    return 1;
  }
  return 0;
}


int handy_awnser_check()
{
  long_delay(5);
  uart_str_complete_0 =0;
  uart_puts("AT\r\r");
  intelli_wait(500,0,"OK");

  uart_str_complete_0 =0;
  uart_puts("AT\r\r");
  intelli_wait(800,0,"OK");

  uart_str_complete_0 =0;
  uart_puts("ATE0\r"); // Repeat Funktion im Handy ausschalten
  if (intelli_wait(1000,0,"OK") == 1)
  {
    return 1;
  }
  
  uart_str_complete_0 =0;
  uart_puts("AT\r\r");
  if (intelli_wait(600,0,"OK") == 1)
  {
    return 1;
  }
  return 0;
}


//*************************************************************************************************************************************************************
//**************************************************************************************************************************************************************************

int main(void)
{
  ports_init();
  uart_init();
  sei();
  while(1)
  {
    while (handy_awnser_check() == 0)
    {
      uart_puts("\r\rHandy nicht erreicht,\r Kommunikationsfehler\r");
      uart_puts("OK erwartet.\r\r");
      // sleep
      PORTB &= ~( 1 << PB0 );  // Toggle PB0 LED Y
      long_delay(15000);       // Warten
      PORTB |= ( 1 << PB0 );  // Toggle PB0 LED Y
    }
    delete_all_sms();
    do
    {
      // sleep
      PORTB &= ~( 1 << PB0 );  // Toggle PB0 LED Y
      long_delay(10000);       // Warten in 10 ms
      PORTB |= ( 1 << PB0 );  // Toggle PB0 LED Y
    }
    while(check_sms() == 1);
  }
  return 0;
}



